@@ -49,7 +49,10 @@ class NoneIsAllowedError(PydanticTypeError):
 
 class WrongConstantError(PydanticValueError):
     code = 'const'
-    msg_template = 'expected constant value {const!r}'
+
+    def __str__(self) -> str:
+        permitted = ', '.join(repr(v) for v in self.ctx['permitted'])  # type: ignore
+        return f'unexpected value; permitted: {permitted}'
 
 
 class BytesError(PydanticTypeError):
@@ -83,7 +86,9 @@ class UrlRegexError(PydanticValueError):
 
 
 class EnumError(PydanticTypeError):
-    msg_template = 'value is not a valid enumeration member'
+    def __str__(self) -> str:
+        permitted = ', '.join(repr(v.value) for v in self.ctx['enum_type'])  # type: ignore
+        return f'value is not a valid enumeration member; permitted: {permitted}'
 
 
 class IntegerError(PydanticTypeError):
@@ -146,6 +151,22 @@ class TupleLengthError(PydanticValueError):
         super().__init__(actual_length=actual_length, expected_length=expected_length)
 
 
+class ListMinLengthError(PydanticValueError):
+    code = 'list.min_items'
+    msg_template = 'ensure this value has at least {limit_value} items'
+
+    def __init__(self, *, limit_value: int) -> None:
+        super().__init__(limit_value=limit_value)
+
+
+class ListMaxLengthError(PydanticValueError):
+    code = 'list.max_items'
+    msg_template = 'ensure this value has at most {limit_value} items'
+
+    def __init__(self, *, limit_value: int) -> None:
+        super().__init__(limit_value=limit_value)
+
+
 class AnyStrMinLengthError(PydanticValueError):
     code = 'any_str.min_length'
     msg_template = 'ensure this value has at least {limit_value} characters'
@@ -333,3 +354,11 @@ class IPv4InterfaceError(PydanticValueError):
 
 class IPv6InterfaceError(PydanticValueError):
     msg_template = 'value is not a valid IPv6 interface'
+
+
+class ColorError(PydanticValueError):
+    msg_template = 'value is not a valid color: {reason}'
+
+
+class StrictBoolError(PydanticValueError):
+    msg_template = 'value is not a valid boolean'
