@@ -37,6 +37,7 @@ from .utils import (
     AnyType,
     ForwardRef,
     GetterDict,
+    ValueItems,
     change_exception,
     is_classvar,
     resolve_annotations,
@@ -58,7 +59,9 @@ if TYPE_CHECKING:  # pragma: no cover
     SetStr = Set[str]
     ListStr = List[str]
     Model = TypeVar('Model', bound='BaseModel')
-
+    IntStr = Union[int, str]
+    SetIntStr = Set[IntStr]
+    DictIntStrAny = Dict[IntStr, Any]
 
 try:
     import cython  # type: ignore
@@ -94,17 +97,18 @@ class BaseConfig:
     json_encoders: Dict[AnyType, AnyCallable] = {}
     orm_mode: bool = False
     alias_generator: Optional[Callable[[str], str]] = None
+    keep_untouched: Tuple[type, ...] = ()
 
     @classmethod
     def get_field_schema(cls, name: str) -> Dict[str, str]:
         field_config = cls.fields.get(name) or {}
         if isinstance(field_config, str):
             field_config = {'alias': field_config}
-        elif not field_config and cls.alias_generator:
+        elif cls.alias_generator and 'alias' not in field_config:
             alias = cls.alias_generator(name)
             if not isinstance(alias, str):
                 raise TypeError(f'Config.alias_generator must return str, not {type(alias)}')
-            field_config = {'alias': alias}
+            field_config['alias'] = alias
         return field_config
 
 
@@ -166,7 +170,7 @@ def validate_custom_root_type(fields: Dict[str, Field]) -> None:
         raise TypeError('custom root type cannot allow mapping')
 
 
-TYPE_BLACKLIST = FunctionType, property, type, classmethod, staticmethod
+UNTOUCHED_TYPES = FunctionType, property, type, classmethod, staticmethod
 
 
 class MetaModel(ABCMeta):
@@ -214,10 +218,11 @@ class MetaModel(ABCMeta):
                         config=config,
                     )
 
+            untouched_types = UNTOUCHED_TYPES + config.keep_untouched
             for var_name, value in namespace.items():
                 if (
                     is_valid_field(var_name)
-                    and (annotations.get(var_name) == PyObject or not isinstance(value, TYPE_BLACKLIST))
+                    and (annotations.get(var_name) == PyObject or not isinstance(value, untouched_types))
                     and var_name not in class_vars
                 ):
                     validate_field_name(bases, var_name)
@@ -263,13 +268,14 @@ class BaseModel(metaclass=MetaModel):
     Config = BaseConfig
     __slots__ = ('__values__', '__fields_set__')
 
-    def __init__(self, **data: Any) -> None:
+    def __init__(__pydantic_self__, **data: Any) -> None:
+        # Uses something other than `self` the first arg to allow "self" as a settable attribute
         if TYPE_CHECKING:  # pragma: no cover
-            self.__values__: Dict[str, Any] = {}
-            self.__fields_set__: 'SetStr' = set()
-        values, fields_set, _ = validate_model(self, data)
-        object.__setattr__(self, '__values__', values)
-        object.__setattr__(self, '__fields_set__', fields_set)
+            __pydantic_self__.__values__: Dict[str, Any] = {}
+            __pydantic_self__.__fields_set__: 'SetStr' = set()
+        values, fields_set, _ = validate_model(__pydantic_self__, data)
+        object.__setattr__(__pydantic_self__, '__values__', values)
+        object.__setattr__(__pydantic_self__, '__fields_set__', fields_set)
 
     @no_type_check
     def __getattr__(self, name):
@@ -303,7 +309,12 @@ class BaseModel(metaclass=MetaModel):
         object.__setattr__(self, '__fields_set__', state['__fields_set__'])
 
     def dict(
-        self, *, include: 'SetStr' = None, exclude: 'SetStr' = None, by_alias: bool = False, skip_defaults: bool = False
+        self,
+        *,
+        include: Union['SetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
+        by_alias: bool = False,
+        skip_defaults: bool = False,
     ) -> 'DictStrAny':
         """
         Generate a dictionary representation of the model, optionally specifying which fields to include or exclude.
@@ -311,13 +322,17 @@ class BaseModel(metaclass=MetaModel):
         get_key = self._get_key_factory(by_alias)
         get_key = partial(get_key, self.fields)
 
-        return_keys = self._calculate_keys(include=include, exclude=exclude, skip_defaults=skip_defaults)
-        if return_keys is None:
-            return {get_key(k): v for k, v in self._iter(by_alias=by_alias, skip_defaults=skip_defaults)}
-        else:
-            return {
-                get_key(k): v for k, v in self._iter(by_alias=by_alias, skip_defaults=skip_defaults) if k in return_keys
-            }
+        allowed_keys = self._calculate_keys(include=include, exclude=exclude, skip_defaults=skip_defaults)
+        return {
+            get_key(k): v
+            for k, v in self._iter(
+                by_alias=by_alias,
+                allowed_keys=allowed_keys,
+                include=include,
+                exclude=exclude,
+                skip_defaults=skip_defaults,
+            )
+        }
 
     def _get_key_factory(self, by_alias: bool) -> Callable[..., str]:
         if by_alias:
@@ -328,8 +343,8 @@ class BaseModel(metaclass=MetaModel):
     def json(
         self,
         *,
-        include: 'SetStr' = None,
-        exclude: 'SetStr' = None,
+        include: Union['SetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
         by_alias: bool = False,
         skip_defaults: bool = False,
         encoder: Optional[Callable[[Any], Any]] = None,
@@ -358,12 +373,7 @@ class BaseModel(metaclass=MetaModel):
                 except (TypeError, ValueError) as e:
                     exc = TypeError(f'{cls.__name__} expected dict not {type(obj).__name__}')
                     raise ValidationError([ErrorWrapper(exc, loc='__obj__')]) from e
-
-        m = cls.__new__(cls)
-        values, fields_set, _ = validate_model(m, obj)
-        object.__setattr__(m, '__values__', values)
-        object.__setattr__(m, '__fields_set__', fields_set)
-        return m
+        return cls(**obj)
 
     @classmethod
     def parse_raw(
@@ -421,8 +431,8 @@ class BaseModel(metaclass=MetaModel):
     def copy(
         self: 'Model',
         *,
-        include: 'SetStr' = None,
-        exclude: 'SetStr' = None,
+        include: Union['SetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
         update: 'DictStrAny' = None,
         deep: bool = False,
     ) -> 'Model':
@@ -440,11 +450,23 @@ class BaseModel(metaclass=MetaModel):
             # skip constructing values if no arguments are passed
             v = self.__values__
         else:
-            return_keys = self._calculate_keys(include=include, exclude=exclude, skip_defaults=False)
-            if return_keys:
-                v = {**{k: v for k, v in self.__values__.items() if k in return_keys}, **(update or {})}
-            else:
+            allowed_keys = self._calculate_keys(include=include, exclude=exclude, skip_defaults=False, update=update)
+            if allowed_keys is None:
                 v = {**self.__values__, **(update or {})}
+            else:
+                v = {
+                    **dict(
+                        self._iter(
+                            to_dict=False,
+                            by_alias=False,
+                            include=include,
+                            exclude=exclude,
+                            skip_defaults=False,
+                            allowed_keys=allowed_keys,
+                        )
+                    ),
+                    **(update or {}),
+                }
 
         if deep:
             v = deepcopy(v)
@@ -477,31 +499,70 @@ class BaseModel(metaclass=MetaModel):
     @classmethod
     def validate(cls: Type['Model'], value: Any) -> 'Model':
         if isinstance(value, dict):
-            return cls.parse_obj(value)
+            return cls(**value)
         elif isinstance(value, cls):
             return value.copy()
         elif cls.__config__.orm_mode:
             return cls.from_orm(value)
         else:
             with change_exception(DictError, TypeError, ValueError):
-                return cls.parse_obj(value)
+                return cls(**dict(value))
 
     @classmethod
     def _decompose_class(cls: Type['Model'], obj: Any) -> GetterDict:
         return GetterDict(obj)
 
     @classmethod
-    def _get_value(cls, v: Any, by_alias: bool, skip_defaults: bool) -> Any:
+    @no_type_check
+    def _get_value(
+        cls,
+        v: Any,
+        to_dict: bool,
+        by_alias: bool,
+        include: Optional[Union['SetIntStr', 'DictIntStrAny']],
+        exclude: Optional[Union['SetIntStr', 'DictIntStrAny']],
+        skip_defaults: bool,
+    ) -> Any:
+
         if isinstance(v, BaseModel):
-            return v.dict(by_alias=by_alias, skip_defaults=skip_defaults)
-        elif isinstance(v, list):
-            return [cls._get_value(v_, by_alias=by_alias, skip_defaults=skip_defaults) for v_ in v]
-        elif isinstance(v, dict):
-            return {k_: cls._get_value(v_, by_alias=by_alias, skip_defaults=skip_defaults) for k_, v_ in v.items()}
-        elif isinstance(v, set):
-            return {cls._get_value(v_, by_alias=by_alias, skip_defaults=skip_defaults) for v_ in v}
-        elif isinstance(v, tuple):
-            return tuple(cls._get_value(v_, by_alias=by_alias, skip_defaults=skip_defaults) for v_ in v)
+            if to_dict:
+                return v.dict(by_alias=by_alias, skip_defaults=skip_defaults, include=include, exclude=exclude)
+            else:
+                return v.copy(include=include, exclude=exclude)
+
+        value_exclude = ValueItems(v, exclude) if exclude else None
+        value_include = ValueItems(v, include) if include else None
+
+        if isinstance(v, dict):
+            return {
+                k_: cls._get_value(
+                    v_,
+                    to_dict=to_dict,
+                    by_alias=by_alias,
+                    skip_defaults=skip_defaults,
+                    include=value_include and value_include.for_element(k_),
+                    exclude=value_exclude and value_exclude.for_element(k_),
+                )
+                for k_, v_ in v.items()
+                if (not value_exclude or not value_exclude.is_excluded(k_))
+                and (not value_include or value_include.is_included(k_))
+            }
+
+        elif isinstance(v, (list, set, tuple)):
+            return type(v)(
+                cls._get_value(
+                    v_,
+                    to_dict=to_dict,
+                    by_alias=by_alias,
+                    skip_defaults=skip_defaults,
+                    include=value_include and value_include.for_element(i),
+                    exclude=value_exclude and value_exclude.for_element(i),
+                )
+                for i, v_ in enumerate(v)
+                if (not value_exclude or not value_exclude.is_excluded(i))
+                and (not value_include or value_include.is_included(i))
+            )
+
         else:
             return v
 
@@ -521,14 +582,37 @@ class BaseModel(metaclass=MetaModel):
         """
         yield from self._iter()
 
-    def _iter(self, by_alias: bool = False, skip_defaults: bool = False) -> 'TupleGenerator':
+    def _iter(
+        self,
+        to_dict: bool = True,
+        by_alias: bool = False,
+        allowed_keys: Optional['SetStr'] = None,
+        include: Union['SetIntStr', 'DictIntStrAny'] = None,
+        exclude: Union['SetIntStr', 'DictIntStrAny'] = None,
+        skip_defaults: bool = False,
+    ) -> 'TupleGenerator':
+
+        value_exclude = ValueItems(self, exclude) if exclude else None
+        value_include = ValueItems(self, include) if include else None
+
         for k, v in self.__values__.items():
-            yield k, self._get_value(v, by_alias=by_alias, skip_defaults=skip_defaults)
+            if allowed_keys is None or k in allowed_keys:
+                yield k, self._get_value(
+                    v,
+                    to_dict=to_dict,
+                    by_alias=by_alias,
+                    include=value_include and value_include.for_element(k),
+                    exclude=value_exclude and value_exclude.for_element(k),
+                    skip_defaults=skip_defaults,
+                )
 
     def _calculate_keys(
-        self, include: 'SetStr' = None, exclude: Optional['SetStr'] = None, skip_defaults: bool = False
+        self,
+        include: Optional[Union['SetIntStr', 'DictIntStrAny']],
+        exclude: Optional[Union['SetIntStr', 'DictIntStrAny']],
+        skip_defaults: bool,
+        update: Optional['DictStrAny'] = None,
     ) -> Optional['SetStr']:
-
         if include is None and exclude is None and skip_defaults is False:
             return None
 
@@ -537,11 +621,20 @@ class BaseModel(metaclass=MetaModel):
         else:
             keys = set(self.__values__.keys())
 
-        if include:
-            keys &= include
+        if include is not None:
+            if isinstance(include, dict):
+                keys &= include.keys()
+            else:
+                keys &= include
+
+        if update:
+            keys -= update.keys()
 
         if exclude:
-            keys -= exclude
+            if isinstance(exclude, dict):
+                keys -= {k for k, v in exclude.items() if v is ...}
+            else:
+                keys -= exclude
 
         return keys
 
