@@ -29,6 +29,7 @@ from .utils import (
     AnyCallable,
     AnyType,
     ForwardRef,
+    almost_equal_floats,
     change_exception,
     display_as_type,
     is_callable_type,
@@ -121,9 +122,10 @@ def float_validator(v: Any) -> float:
 
 def number_multiple_validator(v: 'Number', field: 'Field') -> 'Number':
     field_type: ConstrainedNumber = field.type_  # type: ignore
-    if field_type.multiple_of is not None and v % field_type.multiple_of != 0:  # type: ignore
-        raise errors.NumberNotMultipleError(multiple_of=field_type.multiple_of)
-
+    if field_type.multiple_of is not None:
+        mod = float(v) / float(field_type.multiple_of) % 1
+        if not almost_equal_floats(mod, 0.0) and not almost_equal_floats(mod, 1.0):
+            raise errors.NumberNotMultipleError(multiple_of=field_type.multiple_of)
     return v
 
 
@@ -217,9 +219,10 @@ def set_validator(v: Any) -> Set[Any]:
 
 
 def enum_validator(v: Any, field: 'Field', config: 'BaseConfig') -> Enum:
-    with change_exception(errors.EnumError, ValueError):
+    try:
         enum_v = field.type_(v)
-
+    except ValueError:
+        raise errors.EnumError(enum_type=field.type_)
     return enum_v.value if config.use_enum_values else enum_v
 
 
@@ -472,7 +475,9 @@ def find_validators(  # noqa: C901 (ignore complexity)
     if config.arbitrary_types_allowed:
         yield make_arbitrary_type_validator(type_)
     else:
-        raise RuntimeError(f'no validator found for {type_}')
+        raise RuntimeError(
+            f'no validator found for {type_} see `keep_untouched` or `arbitrary_types_allowed` in Config'
+        )
 
 
 def _find_supertype(type_: AnyType) -> Optional[AnyType]:
